
/** Show a list of minimized windows on the desktop

6/8/2012 : Added support for multiple monitors (show on primary monitor)
6/8/2012 : Limit the size of the label to 75% of the primary monitor

**/

// TODO
// * Possibly distinguish between windows on different workspaces
//   (currently shows all minimized windows - regardless of workspace)
// * Need to dynamically cope with change of 'Desktop managed by...' setting
//   (currently requires restart of the extension (or Gnome))
// * Possibly add click-through of items in list to switch to that window


const St = imports.gi.St;
const Main = imports.ui.main;
const Shell = imports.gi.Shell;
const Meta = imports.gi.Meta;
const Lang = imports.lang;

var _text = null;
var _desktop = null;
var _restackedId = null;

/**
* Build a string with the application and title of all minimized windows and
* position it at the bottom left corner of the primary screen.
*/
function _show_text(source)
{
    //======================================//
    // Remove the previous text if present. //
    //======================================//
    if (_text) {
    	_text.destroy();
    	_text = null;
    }

    //========================================================//
    // Build a string containing a list of minimized windows. //
    //========================================================//
    let string = null;
    let _tracker = Shell.WindowTracker.get_default();
    let windows = global.get_window_actors();

    for (let i = 0; i < windows.length; i++) {
    	if (windows[i].get_meta_window().minimized)
	{
		let name = _tracker.get_window_app(windows[i].get_meta_window()).get_name();
		let title = windows[i].get_meta_window().get_title();
		if (string == null)
		{
			string = name+" - "+title;
		}
		else
		{
			string = string + "\n"+name+" - "+title;
		}

	}

    }

    //==========================================================//
    // If the string is not null, add the label to the desktop. //
    //==========================================================//
    if (string != null)
    {
    	_text = new St.Label({ style_class: 'minimisedWindows-label', text: string });

	if (_desktop == null)
	{
		//===================================================//
		// Search for the DESKTOP window in the window list. //
		//===================================================//
		let i;
		for (i=0; i<windows.length; i++)
		{
			if (windows[i].get_meta_window().get_window_type() == Meta.WindowType.DESKTOP)
			{
				_desktop = windows[i];
				break;
			}
		}
		if (i >= windows.length)
		{
			global.log("MinimizedWindowReminder failed to find DESKTOP window");
			//======================================//
			//If we didn't find it then the desktop //
			//isn't managed by the File Manager so  //
			//we'll add it to global.window_group   //
			//======================================//
			_desktop = global.window_group;
		}
	}

	//==================================================================//
	// Add the window to the DESKTOP and set it's position and opacity. //
	//==================================================================//
    	_desktop.add_actor(_text);
    	_text.opacity = 128;
	let monitor = Main.layoutManager.primaryMonitor;

	//--------------------------------------------//
	// If label is more than 75% of monitor width //
	// then reduce it to 75% of monitor width     //
	//--------------------------------------------//
	if (_text.width > (monitor.width*3/4))
	{
		_text.width = monitor.width*3/4;
	}

	//======================================================//
	// Position near bottom-right corner of primary monitor //
	//======================================================//
    	_text.set_position( monitor.width - _text.width - 16 + monitor.x,
                	    monitor.height - _text.height - 16 + monitor.y);

	//================================================//
	// If it's added to window_group then ensure it's //
	// above the lowest window (ie, the background)   //
	//================================================//
	if (_desktop == global.window_group) {
		_text.raise(global.background_actor);
	}
	else
	{
		//=========================================================//
		// May be only part of desktop being being managed by File //
		// Manager so adjust for the File Manager window position  //
		//=========================================================//
		_text.x = _text.x - _desktop.x;
		_text.y = _text.y - _desktop.y;
	}

    }
}

function init() {
}

function enable() {

	_restackedId = global.screen.connect('restacked', Lang.bind(this, _restacked));

	_show_text("enabled");
}

function _restacked() {

	_show_text("restacked");
}


function disable() {

	//=================================//
	// Unhook from the Window Manager. //
	//=================================//
	global.screen.disconnect(_restackedId);

	//=============================//
	// Remove the text if present. //
	//=============================//
	if (_text) {
		_text.destroy();
		_text = null;
	}

}
